@@ -1,10 +1,12 @@
 import logging
+import optparse
 import os
 import re
 import sys
 import ConfigParser
 from optparse import OptionParser
 from nose.util import absdir, tolist
+from nose.plugins.manager import NoPlugins
 from warnings import warn
 
 log = logging.getLogger(__name__)
@@ -19,6 +21,117 @@ config_files = [
     "~/nose.cfg"
     ]
 
+# plaforms on which the exe check defaults to off
+# Windows and IronPython
+exe_allowed_platforms = ('win32', 'cli')
+
+
+class NoSuchOptionError(Exception):
+    def __init__(self, name):
+        Exception.__init__(self, name)
+        self.name = name
+
+
+class ConfigError(Exception):
+    pass
+
+
+class ConfiguredDefaultsOptionParser(object):
+    """
+    Handler for options from commandline and config files.
+    """
+    def __init__(self, parser, config_section, error=None, file_error=None):
+        self._parser = parser
+        self._config_section = config_section
+        if error is None:
+            error = self._parser.error
+        self._error = error
+        if file_error is None:
+            file_error = lambda msg, **kw: error(msg)
+        self._file_error = file_error
+
+    def _configTuples(self, cfg, filename):
+        config = []
+        if self._config_section in cfg.sections():
+            for name, value in cfg.items(self._config_section):
+                config.append((name, value, filename))
+        return config
+
+    def _readFromFilenames(self, filenames):
+        config = []
+        for filename in filenames:
+            cfg = ConfigParser.RawConfigParser()
+            try:
+                cfg.read(filename)
+            except ConfigParser.Error, exc:
+                raise ConfigError("Error reading config file %r: %s" %
+                                  (filename, str(exc)))
+            config.extend(self._configTuples(cfg, filename))
+        return config
+
+    def _readFromFileObject(self, fh):
+        cfg = ConfigParser.RawConfigParser()
+        try:
+            filename = fh.name
+        except AttributeError:
+            filename = '<???>'
+        try:
+            cfg.readfp(fh)
+        except ConfigParser.Error, exc:
+            raise ConfigError("Error reading config file %r: %s" %
+                              (filename, str(exc)))
+        return self._configTuples(cfg, filename)
+
+    def _readConfiguration(self, config_files):
+        try:
+            config_files.readline
+        except AttributeError:
+            filename_or_filenames = config_files
+            if isinstance(filename_or_filenames, basestring):
+                filenames = [filename_or_filenames]
+            else:
+                filenames = filename_or_filenames
+            config = self._readFromFilenames(filenames)
+        else:
+            fh = config_files
+            config = self._readFromFileObject(fh)
+        return config
+
+    def _processConfigValue(self, name, value, values, parser):
+        opt_str = '--' + name
+        option = parser.get_option(opt_str)
+        if option is None:
+            raise NoSuchOptionError(name)
+        else:
+            option.process(opt_str, value, values, parser)
+
+    def _applyConfigurationToValues(self, parser, config, values):
+        for name, value, filename in config:
+            if name in option_blacklist:
+                continue
+            try:
+                self._processConfigValue(name, value, values, parser)
+            except NoSuchOptionError, exc:
+                self._file_error(
+                    "Error reading config file %r: "
+                    "no such option %r" % (filename, exc.name),
+                    name=name, filename=filename)
+            except optparse.OptionValueError, exc:
+                msg = str(exc).replace('--' + name, repr(name), 1)
+                self._file_error("Error reading config file %r: "
+                                 "%s" % (filename, msg),
+                                 name=name, filename=filename)
+
+    def parseArgsAndConfigFiles(self, args, config_files):
+        values = self._parser.get_default_values()
+        try:
+            config = self._readConfiguration(config_files)
+        except ConfigError, exc:
+            self._error(str(exc))
+        else:
+            self._applyConfigurationToValues(self._parser, config, values)
+        return self._parser.parse_args(args, values)
+
 
 class Config(object):
     """nose configuration.
@@ -37,7 +150,7 @@ class Config(object):
       self.exclude = None
       self.getTestCaseNamesCompat = False
       self.includeExe = env.get('NOSE_INCLUDE_EXE',
-                                sys.platform == 'win32')
+                                sys.platform in exe_allowed_platforms)
       self.ignoreFiles = (re.compile(r'^\.'),
                           re.compile(r'^_'),
                           re.compile(r'^setup\.py$')
@@ -53,7 +166,7 @@ class Config(object):
       self.stopOnError = env.get('NOSE_STOP', False)
       self.stream = sys.stderr
       self.testNames = ()
-      self.verbosity = int(env.get('NOSE_VERBOSE', 0))
+      self.verbosity = int(env.get('NOSE_VERBOSE', 1))
       self.where = ()
       self.workingDir = None   
     """
@@ -71,7 +184,7 @@ class Config(object):
         self.exclude = None
         self.getTestCaseNamesCompat = False
         self.includeExe = env.get('NOSE_INCLUDE_EXE',
-                                  sys.platform == 'win32')
+                                  sys.platform in exe_allowed_platforms)
         self.ignoreFiles = (re.compile(r'^\.'),
                             re.compile(r'^_'),
                             re.compile(r'^setup\.py$')
@@ -90,6 +203,9 @@ class Config(object):
         self.verbosity = int(env.get('NOSE_VERBOSE', 1))
         self.where = ()
         self.workingDir = os.getcwd()
+        self.traverseNamespace = False
+        self.firstPackageWins = False
+        self.parserClass = OptionParser
         
         self._default = self.__dict__.copy()
         self.update(kw)
@@ -106,6 +222,20 @@ class Config(object):
                                           for k in keys ])
     __str__ = __repr__
 
+    def _parseArgs(self, argv, cfg_files):
+        def warn_sometimes(msg, name=None, filename=None):
+            if (hasattr(self.plugins, 'excludedOption') and
+                self.plugins.excludedOption(name)):
+                msg = ("Option %r in config file %r ignored: "
+                       "excluded by runtime environment" %
+                       (name, filename))
+                warn(msg, RuntimeWarning)
+            else:
+                raise ConfigError(msg)
+        parser = ConfiguredDefaultsOptionParser(
+            self.getParser(), self.configSection, file_error=warn_sometimes)
+        return parser.parseArgsAndConfigFiles(argv[1:], cfg_files)
+
     def configure(self, argv=None, doc=None):
         """Configure the nose running environment. Execute configure before
         collecting tests with nose.TestCollector to enable output capture and
@@ -114,20 +244,17 @@ class Config(object):
         env = self.env
         if argv is None:
             argv = sys.argv
-        if hasattr(self, 'files'):
-            argv = self.loadConfig(self.files, argv)
-        parser = self.getParser(doc)        
-        options, args = parser.parse_args(argv)
+
+        cfg_files = getattr(self, 'files', [])
+        options, args = self._parseArgs(argv, cfg_files)
         # If -c --config has been specified on command line,
-        # load those config files to create a new argv set and reparse
-        if options.files:
-            argv = self.loadConfig(options.files, argv)
-            options, args = parser.parse_args(argv)
+        # load those config files and reparse
+        if getattr(options, 'files', []):
+            options, args = self._parseArgs(argv, options.files)
 
         self.options = options
-        tests = args[1:]
-        if tests:
-            self.testNames = tests
+        if args:
+            self.testNames = args
         if options.testNames is not None:
             self.testNames.extend(tolist(options.testNames))
 
@@ -146,9 +273,11 @@ class Config(object):
         self.stopOnError = options.stopOnError
         self.verbosity = options.verbosity
         self.includeExe = options.includeExe
+        self.traverseNamespace = options.traverseNamespace
         self.debug = options.debug
         self.debugLog = options.debugLog
         self.loggingConfig = options.loggingConfig
+        self.firstPackageWins = options.firstPackageWins
         self.configureLogging()
 
         if options.where is not None:
@@ -193,7 +322,7 @@ class Config(object):
 
         # only add our default handler if there isn't already one there
         # this avoids annoying duplicate log messages.
-        if not logger.handlers:
+        if handler not in logger.handlers:
             logger.addHandler(handler)
 
         # default level    
@@ -258,7 +387,7 @@ class Config(object):
         if self.parser:
             return self.parser
         env = self.env
-        parser = OptionParser(doc)
+        parser = self.parserClass(doc)
         parser.add_option(
             "-V","--version", action="store_true",
             dest="version", default=False,
@@ -275,30 +404,37 @@ class Config(object):
             help="Be more verbose. [NOSE_VERBOSE]")
         parser.add_option(
             "--verbosity", action="store", dest="verbosity",
+            metavar='VERBOSITY',
             type="int", help="Set verbosity; --verbosity=2 is "
             "the same as -v")
         parser.add_option(
-            "-q", "--quiet", action="store_const", const=0, dest="verbosity")
+            "-q", "--quiet", action="store_const", const=0, dest="verbosity",
+            help="Be less verbose")
         parser.add_option(
             "-c", "--config", action="append", dest="files",
+            metavar="FILES",
             help="Load configuration from config file(s). May be specified "
             "multiple times; in that case, all config files will be "
             "loaded and combined")
         parser.add_option(
             "-w", "--where", action="append", dest="where",
+            metavar="WHERE",
             help="Look for tests in this directory. "
             "May be specified multiple times. The first directory passed "
             "will be used as the working directory, in place of the current "
             "working directory, which is the default. Others will be added "
             "to the list of tests to execute. [NOSE_WHERE]"
             )
-        parser.add_option("-m", "--match", "--testmatch", action="store",
-                          dest="testMatch",
-                          help="Use this regular expression to "
-                          "find tests [NOSE_TESTMATCH]",
-                          default=self.testMatchPat)
+        parser.add_option(
+            "-m", "--match", "--testmatch", action="store",
+            dest="testMatch", metavar="REGEX",
+            help="Files, directories, function names, and class names "
+            "that match this regular expression are considered tests.  "
+            "Default: %s [NOSE_TESTMATCH]" % self.testMatchPat,
+            default=self.testMatchPat)
         parser.add_option(
             "--tests", action="store", dest="testNames", default=None,
+            metavar='NAMES',
             help="Run these tests (comma-separated list). This argument is "
             "useful mainly from configuration files; on the command line, "
             "just pass the tests to run as additional arguments with no "
@@ -312,23 +448,28 @@ class Config(object):
             "nose.selector. Separate multiple names with a comma.")
         parser.add_option(
             "--debug-log", dest="debugLog", action="store",
-            default=self.debugLog,
+            default=self.debugLog, metavar="FILE",
             help="Log debug messages to this file "
             "(default: sys.stderr)")
         parser.add_option(
             "--logging-config", "--log-config",
             dest="loggingConfig", action="store",
-            default=self.loggingConfig,
+            default=self.loggingConfig, metavar="FILE",
             help="Load logging config from this file -- bypasses all other"
             " logging config settings.")
         parser.add_option(
             "-e", "--exclude", action="append", dest="exclude",
+            metavar="REGEX",
             help="Don't run tests that match regular "
             "expression [NOSE_EXCLUDE]")
         parser.add_option(
             "-i", "--include", action="append", dest="include",
-            help="Also run tests that match regular "
-            "expression [NOSE_INCLUDE]")
+            metavar="REGEX",
+            help="This regular expression will be applied to files, "
+            "directories, function names, and class names for a chance "
+            "to include additional tests that do not match TESTMATCH.  "
+            "Specify this option multiple times "
+            "to add more regular expressions [NOSE_INCLUDE]")
         parser.add_option(
             "-x", "--stop", action="store_true", dest="stopOnError",
             default=self.stopOnError,
@@ -351,6 +492,16 @@ class Config(object):
             help="DO NOT look for tests in python modules that are "
             "executable. (The default on the windows platform is to "
             "do so.)")
+        parser.add_option(
+            "--traverse-namespace", action="store_true",
+            default=self.traverseNamespace, dest="traverseNamespace",
+            help="Traverse through all path entries of a namespace package")
+        parser.add_option(
+            "--first-package-wins", "--first-pkg-wins", "--1st-pkg-wins",
+            default=False, dest="firstPackageWins",
+            help="nose's importer will normally evict a package from sys."
+            "modules if it sees a package with the same name in a different "
+            "location. Set this option to disable that behavior.")
 
         self.plugins.loadPlugins()
         self.pluginOpts(parser)
@@ -363,47 +514,6 @@ class Config(object):
         """
         return self.getParser(doc).format_help()
 
-    def loadConfig(self, file, argv):
-        """Load config from file (may be filename or file-like object) and
-        push the config into argv.
-        """
-        cfg = ConfigParser.RawConfigParser()
-        try:
-            try:
-                cfg.readfp(file)
-            except AttributeError:
-                # Filename not an fp
-                cfg.read(file)
-        except ConfigParser.Error, e:
-            warn("Error reading config file %s: %s" % (file, e),
-                 RuntimeWarning)
-            return argv
-        if self.configSection not in cfg.sections():
-            return argv
-        file_argv = []
-        for optname in cfg.options(self.configSection):
-            if optname in option_blacklist:
-                continue
-            value = cfg.get(self.configSection, optname)
-            file_argv.extend(self.cfgToArg(optname, value))
-        # Copy the given args and insert args loaded from file
-        # between the program name (first arg) and the rest
-        combined = argv[:]
-        combined[1:1] = file_argv
-        return combined
-
-    def cfgToArg(self, optname, value, tr=None):
-        if tr is not None:
-            optname = tr(optname)
-        argv = []
-        if flag(value):
-            if _bool(value):
-                argv.append('--' + optname)
-        else:
-            argv.append('--' + optname)
-            argv.append(value)
-        return argv
-
     def pluginOpts(self, parser):
         self.plugins.addOptions(parser, self.env)
 
@@ -417,16 +527,6 @@ class Config(object):
         self.__dict__.update(d)
 
 
-class NoPlugins(object):
-    """Plugin 'manager' that includes no plugins and returns None
-    for all calls
-    """
-    def __getattr__(self, call):
-        return self
-
-    def __call__(self, *arg, **kw):
-        return
-
 class NoOptions(object):
     """Options container that returns None for all options.
     """
