@@ -69,11 +69,10 @@ def tbsource(tb, context=6):
     The optional second argument specifies the number of lines of context
     to return, which are centered around the current line.
 
-    NOTE:
-    
-    This is adapted from inspect.py in the python 2.4 standard library, since
-    a bug in the 2.3 version of inspect prevents it from correctly locating
-    source lines in a traceback frame.
+    .. Note ::
+       This is adapted from inspect.py in the python 2.4 standard library, 
+       since a bug in the 2.3 version of inspect prevents it from correctly
+       locating source lines in a traceback frame.
     """
     
     lineno = tb.tb_lineno
@@ -86,7 +85,7 @@ def tbsource(tb, context=6):
         try:
             lines, dummy = inspect.findsource(frame)
         except IOError:
-            lines = index = None
+            lines, index = [''], 0
         else:
             all_lines = lines
             start = max(start, 1)
@@ -101,7 +100,7 @@ def tbsource(tb, context=6):
                     start -= 1
                     lines = all_lines[start:start+context]
     else:
-        lines = index = None
+        lines, index = [''], 0
     log.debug("tbsource lines '''%s''' around index %s", lines, index)
     return (lines, index)    
 
