@@ -1,24 +1,23 @@
 """
 This plugin provides assert introspection. When the plugin is enabled
-and a test failure occurs, the traceback of the failure exception is
-examined and displayed with extra context around the line where the
-exception was raised. Simple variable substitution is also performed
-in the context output to provide more debugging information.
+and a test failure occurs, the traceback is displayed with extra context
+around the line in which the exception was raised. Simple variable 
+substitution is also performed in the context output to provide more
+debugging information.
 """
     
-import os
 from nose.plugins import Plugin
 from nose.inspector import inspect_traceback
 
 class FailureDetail(Plugin):
     """
-    Plugin that provides assert introspection. When a test failure occurs, the
-    traceback of the failure exception is examined and displayed with extra
-    context around the line where the exception was raised.
+    Plugin that provides extra information in tracebacks of test failures.
     """
     score = 600 # before capture
     
-    def options(self, parser, env=os.environ):
+    def options(self, parser, env):
+        """Register commmandline options.
+        """
         parser.add_option(
             "-d", "--detailed-errors", "--failure-detail",
             action="store_true",
@@ -28,6 +27,8 @@ class FailureDetail(Plugin):
             " asserts [NOSE_DETAILED_ERRORS]")
 
     def configure(self, options, conf):
+        """Configure plugin.
+        """
         if not self.can_configure:
             return
         self.enabled = options.detailedErrors
