@@ -11,11 +11,19 @@ reporting.
 import logging
 from unittest import _TextTestResult
 from nose.config import Config
-from nose.util import isclass, odict, ln as _ln # backwards compat
+from nose.util import isclass, ln as _ln # backwards compat
 
 log = logging.getLogger('nose.result')
 
 
+def _exception_detail(exc):
+    # this is what stdlib module traceback does
+    try:
+        return str(exc)
+    except:
+        return '<unprintable %s object>' % type(exc).__name__
+
+
 class TextTestResult(_TextTestResult):
     """Text test result that extends unittest's default test result
     support for a configurable set of errorClasses (eg, Skip,
@@ -45,15 +53,22 @@ class TextTestResult(_TextTestResult):
             exc_info = self._exc_info_to_string(err)
         for cls, (storage, label, isfail) in self.errorClasses.items():
             if isclass(ec) and issubclass(ec, cls):
+                if isfail:
+                    test.passed = False
                 storage.append((test, exc_info))
                 # Might get patched into a streamless result
                 if stream is not None:
                     if self.showAll:
-                        stream.writeln(label)
+                        message = [label]
+                        detail = _exception_detail(err[1])
+                        if detail:
+                            message.append(detail)
+                        stream.writeln(": ".join(message))
                     elif self.dots:
                         stream.write(label[:1])
                 return
         self.errors.append((test, exc_info))
+        test.passed = False
         if stream is not None:
             if self.showAll:
                 self.stream.writeln('ERROR')
@@ -66,7 +81,8 @@ class TextTestResult(_TextTestResult):
         _TextTestResult.printErrors(self)
         for cls in self.errorClasses.keys():
             storage, label, isfail = self.errorClasses[cls]
-            self.printErrorList(label, storage)
+            if isfail:
+                self.printErrorList(label, storage)
         # Might get patched into a result with no config
         if hasattr(self, 'config'):
             self.config.plugins.report(self.stream)
@@ -84,31 +100,38 @@ class TextTestResult(_TextTestResult):
         writeln(self.separator2)
         writeln("Ran %s test%s in %.3fs" % (run, plural, taken))
         writeln()
+
+        summary = {}
+        eckeys = self.errorClasses.keys()
+        eckeys.sort()
+        for cls in eckeys:
+            storage, label, isfail = self.errorClasses[cls]
+            count = len(storage)
+            if not count:
+                continue
+            summary[label] = count
+        if len(self.failures):
+            summary['failures'] = len(self.failures)
+        if len(self.errors):
+            summary['errors'] = len(self.errors)
+
         if not self.wasSuccessful():
-            write("FAILED (")
-            summary = odict()
-            summary['failures'], summary['errors'] = \
-                               map(len, [self.failures, self.errors])
-            for cls in self.errorClasses.keys():
-                storage, label, isfail = self.errorClasses[cls]
-                if not isfail:
-                    continue
-                summary[label] = len(storage)
-            any = False
-            for label, count in summary.items():
-                if not count:
-                    continue
-                if any:
-                    write(", ")
-                write("%s=%s" % (label, count))
-                any = True
+            write("FAILED")
+        else:
+            write("OK")
+        items = summary.items()
+        if items:
+            items.sort()
+            write(" (")
+            write(", ".join(["%s=%s" % (label, count) for
+                             label, count in items]))
             writeln(")")
         else:
-            writeln("OK")
+            writeln()
 
     def wasSuccessful(self):
         """Overrides to check that there are no errors in errorClasses
-        lists that are marked as errors that should cause a run to
+        lists that are marked as errors and should cause a run to
         fail.
         """
         if self.errors or self.failures:
