@@ -3,9 +3,8 @@ Test Suites
 -----------
 
 Provides a LazySuite, which is a suite whose test list is a generator
-function, and ContextSuite, a suite that can run fixtures
-(setup/teardown functions or methods) for the context that contains
-its tests.
+function, and ContextSuite,which can run fixtures (setup/teardown
+functions or methods) for the context that contains its tests.
 
 """
 from __future__ import generators
@@ -18,6 +17,11 @@ from nose.config import Config
 from nose.proxy import ResultProxyFactory
 from nose.util import isclass, resolve_name, try_run
 
+if sys.platform == 'cli':
+    import clr
+    clr.AddReference("IronPython")
+    from IronPython.Runtime.Exceptions import StringException
+
 log = logging.getLogger(__name__)
 #log.setLevel(logging.DEBUG)
 
@@ -111,9 +115,20 @@ class ContextSuite(LazySuite):
     failureException = unittest.TestCase.failureException
     was_setup = False
     was_torndown = False
-
+    classSetup = ('setup_class', 'setup_all', 'setupClass', 'setupAll',
+                     'setUpClass', 'setUpAll')
+    classTeardown = ('teardown_class', 'teardown_all', 'teardownClass',
+                     'teardownAll', 'tearDownClass', 'tearDownAll')
+    moduleSetup = ('setup_module', 'setupModule', 'setUpModule', 'setup',
+                   'setUp')
+    moduleTeardown = ('teardown_module', 'teardownModule', 'tearDownModule',
+                      'teardown', 'tearDown')
+    packageSetup = ('setup_package', 'setupPackage', 'setUpPackage')
+    packageTeardown = ('teardown_package', 'teardownPackage',
+                       'tearDownPackage')
+    
     def __init__(self, tests=(), context=None, factory=None,
-                 config=None, resultProxy=None):
+                 config=None, resultProxy=None, can_split=True):
         log.debug("Context suite for %s (%s) (%s)", tests, context, id(self))
         self.context = context
         self.factory = factory
@@ -122,6 +137,7 @@ class ContextSuite(LazySuite):
         self.config = config
         self.resultProxy = resultProxy
         self.has_run = False
+        self.can_split = can_split
         LazySuite.__init__(self, tests)
 
     def __repr__(self):
@@ -141,6 +157,19 @@ class ContextSuite(LazySuite):
         """Hook for replacing error tuple output
         """
         return sys.exc_info()
+    
+    def _exc_info(self):
+        """Bottleneck to fix up IronPython string exceptions
+        """
+        e = self.exc_info()
+        if sys.platform == 'cli':
+            if isinstance(e[0], StringException):
+                # IronPython throws these StringExceptions, but
+                # traceback checks type(etype) == str. Make a real
+                # string here.
+                e = (str(e[0]), e[1], e[2])
+
+        return e
 
     def run(self, result):
         """Run tests in suite inside of suite fixtures.
@@ -155,7 +184,7 @@ class ContextSuite(LazySuite):
         except KeyboardInterrupt:
             raise
         except:
-            result.addError(self, self.exc_info())
+            result.addError(self, self._exc_info())
             return
         try:
             for test in self._tests:
@@ -173,8 +202,41 @@ class ContextSuite(LazySuite):
             except KeyboardInterrupt:
                 raise
             except:
-                result.addError(self, self.exc_info())
+                result.addError(self, self._exc_info())
 
+    def hasFixtures(self, ctx_callback=None):
+        context = self.context
+        if context is None:
+            return False
+        if self.implementsAnyFixture(context, ctx_callback=ctx_callback):
+            return True
+        # My context doesn't have any, but its ancestors might
+        factory = self.factory
+        if factory:
+            ancestors = factory.context.get(self, [])
+            for ancestor in ancestors:
+                if self.implementsAnyFixture(
+                    ancestor, ctx_callback=ctx_callback):
+                    return True
+        return False
+
+    def implementsAnyFixture(self, context, ctx_callback):
+        if isclass(context):
+            names = self.classSetup + self.classTeardown
+        else:
+            names = self.moduleSetup + self.moduleTeardown
+            if hasattr(context, '__path__'):
+                names += self.packageSetup + self.packageTeardown
+        # If my context has any fixture attribute, I have fixtures
+        fixt = False
+        for m in names:
+            if hasattr(context, m):
+                fixt = True
+                break
+        if ctx_callback is None:
+            return fixt
+        return ctx_callback(context, fixt)
+    
     def setUp(self):
         log.debug("suite %s setUp called, tests: %s", id(self), self._tests)
         if not self:
@@ -211,17 +273,17 @@ class ContextSuite(LazySuite):
         self.config.plugins.startContext(context)
         log.debug("%s setup context %s", self, context)
         if self.factory:
+            if context in self.factory.was_setup:
+                return
             # note that I ran the setup for this context, so that I'll run
             # the teardown in my teardown
             self.factory.was_setup[context] = self
         if isclass(context):
-            names = ('setup_class', 'setup_all', 'setupClass', 'setupAll',
-                     'setUpClass', 'setUpAll')
+            names = self.classSetup
         else:
-            names = ('setup_module', 'setupModule', 'setUpModule', 'setup')
+            names = self.moduleSetup
             if hasattr(context, '__path__'):
-                names = ('setup_package', 'setupPackage',
-                         'setUpPackage') + names
+                names = self.packageSetup + names
         try_run(context, names)
 
     def shortDescription(self):
@@ -265,16 +327,15 @@ class ContextSuite(LazySuite):
     def teardownContext(self, context):
         log.debug("%s teardown context %s", self, context)
         if self.factory:
+            if context in self.factory.was_torndown:
+                return
             self.factory.was_torndown[context] = self
         if isclass(context):
-            names = ('teardown_class', 'teardown_all', 'teardownClass',
-                     'teardownAll', 'tearDownClass', 'tearDownAll')
+            names = self.classTeardown
         else:
-            names = ('teardown_module', 'teardownModule', 'tearDownModule',
-                     'teardown')
+            names = self.moduleTeardown
             if hasattr(context, '__path__'):
-                names = ('teardown_package', 'teardownPackage',
-                         'tearDownPackage') + names
+                names = self.packageTeardown + names
         try_run(context, names)
         self.config.plugins.stopContext(context)
 
@@ -296,7 +357,7 @@ class ContextSuite(LazySuite):
 class ContextSuiteFactory(object):
     """Factory for ContextSuites. Called with a collection of tests,
     the factory decides on a hierarchy of contexts by introspecting
-    the collection or the tests them selves to find the objects
+    the collection or the tests themselves to find the objects
     containing the test objects. It always returns one suite, but that
     suite may consist of a hierarchy of nested suites.
     """
@@ -317,7 +378,7 @@ class ContextSuiteFactory(object):
         self.was_setup = {}
         self.was_torndown = {}
 
-    def __call__(self, tests):
+    def __call__(self, tests, **kw):
         """Return ``ContextSuite`` for tests. ``tests`` may either
         be a callable (in which case the resulting ContextSuite will
         have no parent context and be evaluated lazily) or an
@@ -327,15 +388,15 @@ class ContextSuiteFactory(object):
         outermost suites belonging to the outermost contexts.
         """
         log.debug("Create suite for %s", tests)
-        context = getattr(tests, 'context', None)
+        context = kw.pop('context', getattr(tests, 'context', None))
         log.debug("tests %s context %s", tests, context)
         if context is None:
             tests = self.wrapTests(tests)
             try:
                 context = self.findContext(tests)
             except MixedContextError:
-                return self.makeSuite(self.mixedSuites(tests), None)
-        return self.makeSuite(tests, context)
+                return self.makeSuite(self.mixedSuites(tests), None, **kw)
+        return self.makeSuite(tests, context, **kw)
         
     def ancestry(self, context):
         """Return the ancestry of the context (that is, all of the
@@ -346,6 +407,11 @@ class ContextSuiteFactory(object):
         log.debug("get ancestry %s", context)
         if context is None:
             return
+        # Methods include reference to module they are defined in, we
+        # don't want that, instead want the module the class is in now
+        # (classes are re-ancestored elsewhere).
+        if hasattr(context, 'im_class'):
+            context = context.im_class
         if hasattr(context, '__module__'):
             ancestors = context.__module__.split('.')
         elif hasattr(context, '__name__'):
@@ -374,10 +440,10 @@ class ContextSuiteFactory(object):
                     % (context, ctx))
         return context
 
-    def makeSuite(self, tests, context):
+    def makeSuite(self, tests, context, **kw):
         suite = self.suiteClass(
             tests, context=context, config=self.config, factory=self,
-            resultProxy=self.resultProxy)
+            resultProxy=self.resultProxy, **kw)
         if context is not None:
             self.suites.setdefault(context, []).append(suite)
             self.context.setdefault(suite, []).append(context)
@@ -449,6 +515,8 @@ class ContextSuiteFactory(object):
             log.debug("wrapping %s", test)
             if isinstance(test, Test) or isinstance(test, unittest.TestSuite):
                 wrapped.append(test)
+            elif isinstance(test, ContextList):
+                wrapped.append(self.makeSuite(test, context=test.context))
             else:
                 wrapped.append(
                     Test(test, config=self.config, resultProxy=self.resultProxy)
