@@ -1,19 +1,87 @@
 """Utility functions and classes used by nose internally.
 """
 import inspect
+import itertools
 import logging
 import os
 import re
 import sys
 import types
 import unittest
-from compiler.consts import CO_GENERATOR
 from types import ClassType, TypeType
 
+try:
+    from compiler.consts import CO_GENERATOR
+except ImportError:
+    # IronPython doesn't have a complier module
+    CO_GENERATOR=0x20
+    
 log = logging.getLogger('nose')
 
 ident_re = re.compile(r'^[A-Za-z_][A-Za-z0-9_.]*$')
 class_types = (ClassType, TypeType)
+skip_pattern = r"(?:\.svn)|(?:[^.]+\.py[co])|(?:.*~)|(?:.*\$py\.class)"
+
+try:
+    set()
+    set = set # make from nose.util import set happy
+except NameError:
+    try:
+        from sets import Set as set
+    except ImportError:
+        pass
+
+
+def ls_tree(dir_path="",
+            skip_pattern=skip_pattern,
+            indent="|-- ", branch_indent="|   ",
+            last_indent="`-- ", last_branch_indent="    "):
+    # TODO: empty directories look like non-directory files
+    return "\n".join(_ls_tree_lines(dir_path, skip_pattern,
+                                    indent, branch_indent,
+                                    last_indent, last_branch_indent))
+
+
+def _ls_tree_lines(dir_path, skip_pattern,
+                   indent, branch_indent, last_indent, last_branch_indent):
+    if dir_path == "":
+        dir_path = os.getcwd()
+
+    lines = []
+
+    names = os.listdir(dir_path)
+    names.sort()
+    dirs, nondirs = [], []
+    for name in names:
+        if re.match(skip_pattern, name):
+            continue
+        if os.path.isdir(os.path.join(dir_path, name)):
+            dirs.append(name)
+        else:
+            nondirs.append(name)
+
+    # list non-directories first
+    entries = list(itertools.chain([(name, False) for name in nondirs],
+                                   [(name, True) for name in dirs]))
+    def ls_entry(name, is_dir, ind, branch_ind):
+        if not is_dir:
+            yield ind + name
+        else:
+            path = os.path.join(dir_path, name)
+            if not os.path.islink(path):
+                yield ind + name
+                subtree = _ls_tree_lines(path, skip_pattern,
+                                         indent, branch_indent,
+                                         last_indent, last_branch_indent)
+                for x in subtree:
+                    yield branch_ind + x
+    for name, is_dir in entries[:-1]:
+        for line in ls_entry(name, is_dir, indent, branch_indent):
+            yield line
+    if entries:
+        name, is_dir = entries[-1]
+        for line in ls_entry(name, is_dir, last_indent, last_branch_indent):
+            yield line
 
 
 def absdir(path):
@@ -81,7 +149,7 @@ def file_like(name):
 
 def cmp_lineno(a, b):
     """Compare functions by their line numbers.
-    
+
     >>> cmp_lineno(isgenerator, ispackage)
     -1
     >>> cmp_lineno(ispackage, isgenerator)
@@ -106,10 +174,11 @@ def func_lineno(func):
 
 
 def isclass(obj):
-    """Is obj a class? inspect's isclass is too liberal and returns True
+    """Is obj a class? Inspect's isclass is too liberal and returns True
     for objects that can't be subclasses of anything.
     """
-    return type(obj) in class_types
+    obj_type = type(obj)
+    return obj_type in class_types or issubclass(obj_type, type)
 
 
 def isgenerator(func):
@@ -142,9 +211,31 @@ def ispackage(path):
             for init in ('__init__.py', '__init__.pyc', '__init__.pyo'):
                 if os.path.isfile(os.path.join(path, init)):
                     return True
+            if sys.platform.startswith('java') and \
+                    os.path.isfile(os.path.join(path, '__init__$py.class')):
+                return True
     return False
 
 
+def isproperty(obj):
+    """
+    Is this a property?
+
+    >>> class Foo:
+    ...     def got(self):
+    ...         return 2
+    ...     def get(self):
+    ...         return 1
+    ...     get = property(get)
+
+    >>> isproperty(Foo.got)
+    False
+    >>> isproperty(Foo.get)
+    True
+    """
+    return type(obj) == property
+
+
 def getfilename(package, relativeTo=None):
     """Find the python source file for a package, relative to a
     particular directory (defaults to current working directory if not
@@ -159,13 +250,13 @@ def getfilename(package, relativeTo=None):
         if os.path.exists(filename):
             return filename
     return None
-    
+
 
 def getpackage(filename):
     """
     Find the full dotted package name for a given python source file
     name. Returns None if the file is not a python source file.
-    
+
     >>> getpackage('foo.py')
     'foo'
     >>> getpackage('biff/baf.py')
@@ -316,15 +407,13 @@ def split_test_name(test):
         return (None, None, fn)
 split_test_name.__test__ = False # do not collect
 
-    
+
 def test_address(test):
     """Find the test address for a test, which may be a module, filename,
     class, method or function.
     """
-    try:
+    if hasattr(test, "address"):
         return test.address()
-    except AttributeError:
-        pass
     # type-based polymorphism sucks in general, but I believe is
     # appropriate here
     t = type(test)
@@ -332,8 +421,8 @@ def test_address(test):
     if t == types.ModuleType:
         file = getattr(test, '__file__', None)
         module = getattr(test, '__name__', None)
-        return (file, module, call)
-    if t == types.FunctionType or t in (type, types.ClassType):
+        return (src(file), module, call)
+    if t == types.FunctionType or issubclass(t, type) or t == types.ClassType:
         module = getattr(test, '__module__', None)
         if module is not None:
             m = sys.modules[module]
@@ -341,12 +430,12 @@ def test_address(test):
             if file is not None:
                 file = os.path.abspath(file)
         call = getattr(test, '__name__', None)
-        return (file, module, call)
+        return (src(file), module, call)
     if t == types.InstanceType:
         return test_address(test.__class__)
     if t == types.MethodType:
         cls_adr = test_address(test.im_class)
-        return (cls_adr[0], cls_adr[1],
+        return (src(cls_adr[0]), cls_adr[1],
                 "%s.%s" % (cls_adr[2], test.__name__))
     # handle unittest.TestCase instances
     if isinstance(test, unittest.TestCase):
@@ -360,7 +449,7 @@ def test_address(test):
             method_name = test._TestCase__testMethodName
         except AttributeError:
             method_name = test._testMethodName
-        return (cls_adr[0], cls_adr[1],
+        return (src(cls_adr[0]), cls_adr[1],
                 "%s.%s" % (cls_adr[2], method_name))
     raise TypeError("I don't know what %s is (%s)" % (test, t))
 test_address.__test__ = False # do not collect
@@ -390,20 +479,23 @@ def try_run(obj, names):
                         raise TypeError("Attribute %s of %r is not a python "
                                         "function. Only functions or callables"
                                         " may be used as fixtures." %
-                                        (name, obj))                    
+                                        (name, obj))
                 if len(args):
-                    log.debug("call fixture %s.%s(%s)", obj, name, obj)    
+                    log.debug("call fixture %s.%s(%s)", obj, name, obj)
                     return func(obj)
             log.debug("call fixture %s.%s", obj, name)
             return func()
 
 
 def src(filename):
-    """Find the python source file for a .pyc or .pyo file. Returns the 
-    filename provided if it is not a python source file.
+    """Find the python source file for a .pyc, .pyo or $py.class file on
+    jython. Returns the filename provided if it is not a python source
+    file.
     """
     if filename is None:
         return filename
+    if sys.platform.startswith('java') and filename.endswith('$py.class'):
+        return '.'.join((filename[:-9], 'py'))
     base, ext = os.path.splitext(filename)
     if ext in ('.pyc', '.pyo', '.py'):
         return '.'.join((base, 'py'))
@@ -428,7 +520,7 @@ def match_last(a, b, regex):
         return -1
     return cmp(a, b)
 
-        
+
 def tolist(val):
     """Convert a value that may be a list or a (possibly comma-separated)
     string into a list. The exception: None is returned as None, not [None].
@@ -452,7 +544,7 @@ def tolist(val):
     try:
         return re.split(r'\s*,\s*', val)
     except TypeError:
-        # who knows... 
+        # who knows...
         return list(val)
 
 
@@ -508,7 +600,72 @@ class odict(dict):
         return map(self.get, self._keys)
 
 
+def transplant_func(func, module):
+    """
+    Make a function imported from module A appear as if it is located
+    in module B.
 
+    >>> from pprint import pprint
+    >>> pprint.__module__
+    'pprint'
+    >>> pp = transplant_func(pprint, __name__)
+    >>> pp.__module__
+    'nose.util'
+
+    The original function is not modified.
+
+    >>> pprint.__module__
+    'pprint'
+
+    Calling the transplanted function calls the original.
+
+    >>> pp([1, 2])
+    [1, 2]
+    >>> pprint([1,2])
+    [1, 2]
+
+    """
+    from nose.tools import make_decorator
+    def newfunc(*arg, **kw):
+        return func(*arg, **kw)
+
+    newfunc = make_decorator(func)(newfunc)
+    newfunc.__module__ = module
+    return newfunc
+
+
+def transplant_class(cls, module):
+    """
+    Make a class appear to reside in `module`, rather than the module in which
+    it is actually defined.
+
+    >>> from nose.failure import Failure
+    >>> Failure.__module__
+    'nose.failure'
+    >>> Nf = transplant_class(Failure, __name__)
+    >>> Nf.__module__
+    'nose.util'
+    >>> Nf.__name__
+    'Failure'
+
+    """
+    class C(cls):
+        pass
+    C.__module__ = module
+    C.__name__ = cls.__name__
+    return C
+
+
+def safe_str(val, encoding='utf-8'):
+    try:
+        return str(val)
+    except UnicodeEncodeError:
+        if isinstance(val, Exception):
+            return ' '.join([safe_str(arg, encoding)
+                             for arg in val])
+        return unicode(val).encode(encoding)
+
+    
 if __name__ == '__main__':
     import doctest
     doctest.testmod()
